using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using TMPro;
using UnityEngine.Events;

namespace ObjectEditor
{
    public struct ModResources
    {
        public static Sprite CloseSprite;
		public static Sprite HideSprite;
		public static Sprite GoUpSprite;
		public static Sprite SwitchSprite;
		public static Sprite RemoveSprite;

		public static void Init()
        {
			CloseSprite = ModAPI.LoadSprite("img/Close.png");
			HideSprite = ModAPI.LoadSprite("img/Hide.png");
			GoUpSprite = ModAPI.LoadSprite("img/GoUp.png");
			SwitchSprite = ModAPI.LoadSprite("img/Switch.png");
			RemoveSprite = ModAPI.LoadSprite("img/Remove.png");
		}
    }

    public class Mod
    {
		public static void Main()
        {
            ModResources.Init();

			ContextMenuButton button = new ContextMenuButton("editButton", "Edit", "Edit", () => ObjectEditorController.Open());
			ModAPI.OnItemSpawned += (sender, args) =>
			{
				foreach (PhysicalBehaviour phys in args.Instance.GetComponentsInChildren<PhysicalBehaviour>())
					if (!phys.ContextMenuOptions.Buttons.Contains(button))
						phys.ContextMenuOptions.Buttons.Add(button);
			};
		}
	}

	public class StayOnScreen : MonoBehaviour
	{
		private Vector4 limits;

		private RectTransform rect;

		private void Awake()
		{
			rect = GetComponent<RectTransform>();
		}

		private void Update()
		{
			if (!gameObject.activeSelf)
				return;

			limits = new Vector4(rect.sizeDelta.x / 2, rect.sizeDelta.y, Screen.width - rect.sizeDelta.x / 2, Screen.height);
			transform.position = new Vector3(Mathf.Clamp(transform.position.x, limits.x, limits.z), Mathf.Clamp(transform.position.y, limits.y, limits.w), 0);
		}
	}

	public class ActionAfterDestroy : MonoBehaviour
	{
		public GameObject go;
		public UnityAction action;

		private void Update()
		{
			if (!go)
				action?.Invoke();
		}

		public void Setup(GameObject go, UnityAction action)
		{
			this.go = go;
			this.action = action;
		}
	}

	public class SwitchGameObjects : MonoBehaviour
	{
		public bool status;
		public Image image;
		public List<GameObject> objects;

		private void Awake()
		{
			GetComponent<Button>().onClick.AddListener(() => Toggle());
		}

		private void Start()
		{
			Apply();
			if (image)
				image.transform.eulerAngles = new Vector3(0, 0, status ? 0 : 180);
		}

		public void Toggle()
		{
			status = !status;
			Apply();
		}

		private void Apply()
		{
			foreach (GameObject obj in objects)
				obj.SetActive(status);
		}

		private void Update()
		{
			if (image)
				image.transform.eulerAngles = new Vector3(0, 0, Mathf.Lerp(image.transform.eulerAngles.z, status ? 0 : 180, 0.1f));
		}
	}

	public class SizeAnimation : MonoBehaviour
	{
		private RectTransform rect;
		public Vector2 targetSize;

		private void Awake()
		{
			rect = GetComponent<RectTransform>();
			targetSize = rect.sizeDelta;
		}
		private void OnEnable()
		{
			rect.sizeDelta = new Vector2(targetSize.x, 0);
		}

		private void Update() => rect.sizeDelta = Vector2.Lerp(rect.sizeDelta, targetSize, 0.05f);
	}

	public class TextColorAnimation : MonoBehaviour
	{
		private TextMeshProUGUI text;
		public Color targetColor;

		private void Awake()
		{
			text = GetComponent<TextMeshProUGUI>();
			targetColor = text.color;
		}

		private void Update() => text.color = Color.Lerp(text.color, targetColor, 0.05f);
	}
}
